#!/usr/local/bin/perl 
# 
#	楽天商品ランキング
#	2010.04.09
#		新規作成
#		楽天商品ランキングAPI (version:2009-04-15) 
#	2010.04.11
#		・このCGIだけで複数のジャンルIDをカバーできる様に仕様変更。
#		・モバイル用にはモバイル用の情報を表示する用に修正。
#		・商品名を切り出す設定が反映されていないのを修正。
#	2010.04.12
#		・携帯用のページを生成するとき文字コードをSHIFT_JISで出力しないようにした。
#
#	2010.09.08
#		・use utf8;するように修正
#		・細々とした修正
#		・軽くするためにCGI.pmを使用しないように修正
#		・楽天ランキング APIをversion:2010-08-05に
#		・restを3.0に
#	2010.09.09
#		・設定を別ファイルに切り離した
#----------------------------------------------------------------------------------
use strict; 
use warnings;
use utf8;
use LWP::Simple; 
use XML::Simple; 
use Encode;
use Encode qw/from_to/; 
use Encode::Guess qw/ascii euc-jp shiftjis/;

my $VERSION		= 'v.0.9.2';

#グローバル変数の定義
our $dir;
our $img_url;
our $developerId;
our $affiliateId;
our $namelen;
our $defGenreId;

#設定ファイルの読み込み
require 'config.cgi';

my $errlog		= "$dir/errlog.cgi";
my $pctxt		= "$dir/pc_";
my $mbtxt		= "$dir/mb_";

# 楽天ウェブサービス 
my $base_url    = 'http://api.rakuten.co.jp/rws/3.0/rest'; 

my $operation   = 'ItemRanking'; 
my $version     = '2010-08-05';

my $shoplink    = "http://hb.afl.rakuten.co.jp/hgc/$affiliateId/?pc=";

my $req_url     = "$base_url?developerId=$developerId&operation=$operation&affiliateId=$affiliateId&version=$version";

#webサービスコピーライト 
my $webservice=<<"EOF"; 
<!-- Rakuten Web Services Attribution Snippet FROM HERE -->
<a href="http://webservice.rakuten.co.jp/" target="_blank">Supported by 楽天ウェブサービス</a>
<!-- Rakuten Web Services Attribution Snippet TO HERE -->
EOF

my %in=();
my %cfg=();
&idecode();
#&loadconfig();

my $genreId = $in{'genreId'} || $defGenreId;


#24時間キャッシュ
if((-e "$pctxt$genreId.html") && ((-M "$pctxt$genreId.html")*86400) < 86400){
	&pcdisp($genreId);
	exit(0);
}

#PC用ページ生成
#	'genreId'=>'401151',	#ジャンルIDの指定
&makehtml({
	'genreId'=>"$genreId",
	'sex'=>'',				#0:男性 1;女性
	'age'=>'',				#年齢 10 20 30 40 50
	'carrier'=>'',			#PC用の情報かモバイル向けの情報か 0:PC 1:mobile
});

sleep(1);

#モバイル用ページ生成
#	'genreId'=>'401151',	#ジャンルIDの指定
&makehtml_mb({
	'genreId'=>"$genreId",
	'sex'=>'',				#0:男性 1;女性
	'age'=>'',				#年齢 10 20 30 40 50
	'carrier'=>'1',			#PC用の情報かモバイル向けの情報か 0:PC 1:mobile
});

&pcdisp($genreId);
exit(0);

#----------------------------------------------------------
#	PC用html 出力
#----------------------------------------------------------
sub makehtml{
	my $ref = shift;
	foreach my $key (keys %{$ref}){
		next if($ref->{$key} eq '');
		$req_url .= "&$key=$ref->{$key}";
	}

	my $data = get($req_url); 
	my $xmlsimple = XML::Simple->new(); 
	my $xml = $xmlsimple->XMLin($data,ForceArray=>['Item']);

	#エラーチェック 
	if($xml->{'header:Header'}->{'Status'} =~ /(ServerError|ClientError|Maintenance)/i){ 
		open(FILE,'>:encoding(utf8)',"$errlog");
		print FILE $req_url,"\n";
		print FILE $data,"\n";
		close(FILE);
		exit(-1);
	}

	my $ary             = \$xml->{Body}{'itemRanking:ItemRanking'}{'Item'}; 
	my $title           = $xml->{Body}{'itemRanking:ItemRanking'}{'title'}; 
	my $lastBuildDate   = $xml->{Body}{'itemRanking:ItemRanking'}{'lastBuildDate'}; 

	#ループ
	foreach my $r (@{$$ary}) { 
		$r->{'itemName'}         = &textcut($r->{'itemName'},$namelen);
		$r->{'affiliateUrl'}   ||='';
		$r->{'smallImageUrl'}  ||= "$img_url/noimg.gif";
		$r->{'mediumImageUrl'} ||= "$img_url/noimg.gif";
	}

	$/ = undef;
	open(FILE,'<:utf8',"$dir/pc_tmpl.txt");
	my $pc_tmpl = <FILE>;
	close(FILE);
	$/ = "\n";

	for(my $i=1;$i<=10;$i++){
		$pc_tmpl =~ s/%itemName:$i%/${$$ary}[$i-1]->{itemName}/g;
		$pc_tmpl =~ s/%affiliateUrl:$i%/${$$ary}[$i-1]->{affiliateUrl}/g;
		$pc_tmpl =~ s/%smallImageUrl:$i%/${$$ary}[$i-1]->{smallImageUrl}/g;
		$pc_tmpl =~ s/%mediumImageUrl:$i%/${$$ary}[$i-1]->{mediumImageUrl}/g;
	}

	open(FILE,'>:encoding(utf8)',"$pctxt$ref->{'genreId'}.html");
	print FILE $pc_tmpl,'by <a href="http://blog.kumacchi.com/" target="_blank">KUMA TYPE</a><br>',$webservice;
	close(FILE);
}

#----------------------------------------------------------
#	モバイル用html 出力
#----------------------------------------------------------
sub makehtml_mb{
	my $ref = shift;
	foreach my $key (keys %{$ref}){
		next if($ref->{$key} eq '');
		$req_url .= "&$key=$ref->{$key}";
	}

	my $data = get($req_url); 
	my $xmlsimple = XML::Simple->new(); 
	my $xml = $xmlsimple->XMLin($data,ForceArray=>['Item']);

	#エラーチェック 
	if($xml->{'header:Header'}->{'Status'} =~ /(ServerError|ClientError|Maintenance)/i){ 
		open(FILE,'>:encoding(utf8)',"$errlog");
		print FILE $req_url,"\n";
		print FILE $data,"\n";
		close(FILE);
		exit(-1);
	}

	my $ary             = \$xml->{Body}{'itemRanking:ItemRanking'}{'Item'}; 
	my $title           = $xml->{Body}{'itemRanking:ItemRanking'}{'title'}; 
	my $lastBuildDate   = $xml->{Body}{'itemRanking:ItemRanking'}{'lastBuildDate'}; 

	#ループ
	foreach my $r (@{$$ary}) { 
		$r->{'itemName'}         = &textcut($r->{'itemName'},$namelen);
		$r->{'affiliateUrl'}   ||='';
		$r->{'smallImageUrl'}  ||= "$img_url/noimg.gif";
		$r->{'mediumImageUrl'} ||= "$img_url/noimg.gif";
	}

	$/ = undef;
	open(FILE,'<:utf8',"$dir/mb_tmpl.txt");
	my $mb_tmpl = <FILE>;
	close(FILE);
	$/ = "\n";

	for(my $i=1;$i<=10;$i++){
		$mb_tmpl =~ s/%itemName:$i%/${$$ary}[$i-1]->{itemName}/g;
		$mb_tmpl =~ s/%affiliateUrl:$i%/${$$ary}[$i-1]->{affiliateUrl}/g;
		$mb_tmpl =~ s/%smallImageUrl:$i%/${$$ary}[$i-1]->{smallImageUrl}/g;
		$mb_tmpl =~ s/%mediumImageUrl:$i%/${$$ary}[$i-1]->{mediumImageUrl}/g;
	}

#	open(FILE,'>:encoding(shiftjis)',"$mbtxt$ref->{'genreId'}.html");
	open(FILE,'>:encoding(utf8)',"$mbtxt$ref->{'genreId'}.html");
	print FILE $mb_tmpl,'by <a href="http://blog.kumacchi.com/mt4i2/mt4i.cgi">KUMA TYPE</a><br>',$webservice;
	close(FILE);
}


#----------------------------------------------------------
#	文字省略
#	textcut (文字列,取り出す文字数)
#----------------------------------------------------------
sub textcut{
	my $str = shift || '';
	my $num = shift || 20;

	my $utf8 = decode_utf8 $str;
	my $str_unit;
	if(length($utf8) > $num){
		$str_unit = substr($utf8,0,$num) . '...';
	}else{
		$str_unit = $utf8;
	}

	$str_unit;
}

#----------------------------------------------------------
#	pc用表示
#----------------------------------------------------------
sub pcdisp{
	my $genreId = shift;
	$/ = undef;
	open(FILE,"$pctxt$genreId.html");
	my $HTML = <FILE>;
	close(FILE);

	print "Content-type: text/html\n\n";
	print << "EOF";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html lang="ja">
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8">
<meta http-equiv="content-style-type" content="text/css">
<link rel="stylesheet" type="text/css" href="style.css">
</head>
<body>
$HTML
</body>
</html>
EOF
	exit(0);
}

#----------------------------------------------------------
# 引数取得
#----------------------------------------------------------
sub idecode{
	my $QUERY_DATA='';
	if($ENV{'REQUEST_METHOD'} eq "POST"){
		read(STDIN,$QUERY_DATA,$ENV{'CONTENT_LENGTH'});
	}else{
		$QUERY_DATA = $ENV{'QUERY_STRING'};
	}
	my @argv = split(/&/,$QUERY_DATA);
	foreach(@argv){
		my ($arg,$val) = split(/=/);
		$val =~ tr/+/ /;
		$val =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("H2",$1)/eg;

		$val =~ s/&/&amp;/g;

		$val =~ s/</&lt;/g;
		$val =~ s/>/&gt;/g;

		$val =~ s/"/&quot;/g;
		$val =~ s/'/&apos;/g;

		$val =~ s/(?:\x0d|\x0a)//g;

		$val =~ s/\t//g;
		$val =~ s/\0//g;

		$in{"$arg"} = $val;
	}
}
